/* Scarab - provides SLIP over a half-duplex link such as infrared or radio.
 *
 * airport.c
 * Commands for Adaptec AirPort (desktop PC IrDA port).
 *
 * (c) Copyright 1997 Robin O'Leary <robin@acm.org>.
 * All Rights Reserved.
 *
 * This program is distributed without any warranty; without even the
 * implied warranty of MERCHANTABILITY or fitness for a particular purpose.
 * See the file LICENSE for details.
 */

#include <unistd.h>	/* write() */
/*#include <sys/ioctl.h>	/* ioctl(), TCIFLUSH */
#include <sys/termios.h>	/* ioctl(), TCIFLUSH */
#include <stdio.h>	/* printf() */
#include <sys/time.h>	/* struct timeval */

#include "main.h"	/* types, globals */
#include "debug.h"	/* for DEBUG_SCARAB */
#include "term.h"	/* for set_termios() */
#include "airport.h"	/* consistency check */


/********************** Private functions ***********************/
 
static int baud_to_airport_code(int baud)
{
	switch(baud)
	{
		case   2400: return 0x10;
		case   9600: return 0x30;
		case  19200: return 0x40;
		case  38400: return 0x50;
		case  57600: return 0x60;
		case 115200: return 0x70;
	}
	fprintf(stderr,"baud_to_airport_code: %d is not a supported baud rate.\n",baud);
	exit(1);
}


static int send_expect(int fd, unsigned char s, unsigned char e, int t)
{
    int i;
    if(debug & DEBUG_SCARAB)
	    fprintf(stderr,"send_expect: sending 0x%02x, waiting %d second%s for 0x%02x\n",s,t,(t==1)?"":"s",e);
    tcflush(fd,TCIFLUSH);
    write(fd,&s,1);
    for(i=0; i<50; i++)
    {
	fd_set read_fds;
	struct timeval timeout;

	FD_ZERO(&read_fds);
	FD_SET(fd, &read_fds);
	timeout.tv_sec=t;
	timeout.tv_usec=0;

	switch(select(fd+1, &read_fds, NULL, NULL, &timeout))
	{
	    case -1:
	    {
		fprintf(stderr,"expect: bad select\n");
		perror("select");
		return -1;
	    }
	    case 0:
	    {
		fprintf(stderr,"expect: timeout waiting for 0x%02x\n",e);
		return -1;
	    }
	    case 1:
	    {
		if(FD_ISSET(fd, &read_fds))
		{
		    unsigned char c;
		    debug_read(fd,&c,1);
		    if(c==e)
		    {
			if(debug & DEBUG_SCARAB)
			    fprintf(stderr,"expect: got character 0x%02x\n",e);
			return 0;
		    }
		    fprintf(stderr,"expect: got 0x%02x not 0x%02x\n",c,e);
		}
		break;
	    }
	}/*switch*/
    }
    if(debug & DEBUG_SCARAB)
	fprintf(stderr,"expect: got too much rubbish\n");
    return -1;
}


int airport_init(int fd, int baud)
{
    char c;
    if(set_termios(fd,9600))
	    return -1;	/* set_termios prints a mesasge */

    if( set_termios_dtr(fd,0)
     || set_termios_rts(fd,0))
	    return -1;
    sleep(3);

    /* Make sure AirPort is in normal mode */
    if( set_termios_dtr(fd,1)
     || set_termios_rts(fd,1))
	    return -1;
    sleep(2);

    if(set_termios_dtr(fd,0)) /* command mode */
	    return -1;	/* set_termios_dtr prints a mesasge */
    if(send_expect(fd,0xff,0xc3,3))
    {
	    set_termios_dtr(fd,1);
	    return -1;	/* expect prints a mesasge */
    }
    usleep(20000);

    c=baud_to_airport_code(baud);
    if(send_expect(fd,c,c,1))	/* should echo */
	    return -1;	/* expect prints a mesasge */
    usleep(20000);

    c|=1;
    if(send_expect(fd,c,c,1))	/* should echo */
	    return -1;	/* expect prints a mesasge */
    usleep(20000);

    if(set_termios_dtr(fd,1)) /* back to data mode */
	    return -1;	/* set_termios_dtr prints a mesasge */

    /* All OK */
    return 0;
}
