/* Scarab - provides SLIP over a half-duplex link such as infrared or radio.
 *
 * debug.c
 * Trace I/O operations.
 *
 * (c) Copyright 1996 Alan Cox, Robin O'Leary <robin@acm.org>.
 * All Rights Reserved.
 *
 * This program is distributed without any warranty; without even the
 * implied warranty of MERCHANTABILITY or fitness for a particular purpose.
 * See the file LICENSE for details.
 */

#include <stdio.h>	/* fprintf() */
#include <ctype.h>	/* isprint() */
#include <unistd.h>	/* read(),write() */

#include "main.h"	/* globals */
#include "debug.h"	/* consistency check */


void debug_packet(const byte data[], int count)
{
    if(debug & DEBUG_BYTE)
    {
	int i;
	int j;
	for(i=0; i<count; i+=16)
	{
		fprintf(stderr,"%04x: ",i);
		for(j=i; j<i+16; j++)
			fprintf(stderr,(j<count)?"%02x ":"   ",data[j]);
		for(j=i; j<(i+16) && (j<count); j++)
			fprintf(stderr,"%c", isprint(data[j])? data[j]:'.');
		fprintf(stderr,"\n");
	}
    }
}

int debug_write(int fd, const char *buf, size_t count)
{
	if(debug & DEBUG_WRITE)
	{
		int result;
		if(count==1)
			fprintf(stderr,"write(%d, %#02x, %d)",fd,*(const byte*)buf,count);
		else
			fprintf(stderr,"write(%d, %p[], %d)",fd,buf,count);
		result=write(fd,buf,count);
		fprintf(stderr,"=%d\n",result);
		if(count>1) debug_packet(buf,count);
		return result;
	}
	return write(fd,buf,count);
}

int debug_read(int fd, char *buf, size_t count)
{
	if(debug & DEBUG_READ)
	{
		int result;
		fprintf(stderr,"read(%d, %p[], %d)",fd,buf,count);
		result=read(fd,buf,count);
		fprintf(stderr,"=%d\n",result);
		if(result>0) debug_packet(buf,result);
		return result;
	}
	return read(fd,buf,count);
}
