/* Scarab - provides SLIP over a half-duplex link such as infrared or radio.
 *
 * term.c
 * Terminal opening and configuring.
 *
 * (c) Copyright 1996 Alan Cox, Robin O'Leary <robin@acm.org>.
 * All Rights Reserved.
 *
 * This program is distributed without any warranty; without even the
 * implied warranty of MERCHANTABILITY or fitness for a particular purpose.
 * See the file LICENSE for details.
 */

#include <stdio.h>	/* fprintf() */
#include <fcntl.h>	/* open() */
#include <unistd.h>	/* close() */
#include <stdio.h>	/* perror() */
#include <termios.h>
#include <sys/ioctl.h>	/* for ioctl() prototype */
#include "main.h"	/* for global variables */
#include "debug.h"	/* for DEBUG_TERM */


/*
 *	Prepare a raw terminal port.
 *
 *	Note that ``baud'' is specified as an int, rather than as a
 *	termios value like ``B9600''.
 */
 
int set_termios(int fd, int baud)
{
	struct termios ts;
	if(debug & DEBUG_TERM)
	{
		fprintf(stderr,"set_termios: fd %d, setting %d baud\n",fd,baud);
	}
	switch(baud)
	{
		case  1200:	baud=B1200;	break;
		case  4800:	baud=B4800;	break;
		case  9600:	baud=B9600;	break;
		case 19200:	baud=B19200;	break;
		case 38400:	baud=B38400;	break;
#ifdef B115200
		case 57600:	baud=B57600;	break;
		case 115200:	baud=B115200;	break;
#endif
#ifdef B1800K
		case 230400:	baud=B225K;	break;
		case 460800:	baud=B450K;	break;
		case 921600:	baud=B900K;	break;
		case 1843200:	baud=B1800K;	break;
#endif
		default:
			fprintf(stderr,"%s: unsupported baud rate %d\n",
				program_name,baud);
			return -1;
	}
	if(tcgetattr(fd,&ts))
	{
		fprintf(stderr,"%s: can't get attributes of terminal\n",
			program_name);
		perror("tcgetattr");
		return -1;
	}
	ts.c_iflag=IGNBRK;
	ts.c_oflag=0;
	/*ts.c_cflag&=~(CBAUD|PARENB|HUPCL|CSIZE|CRTSCTS|);*/
	ts.c_cflag=CLOCAL|CS8|baud;
	ts.c_lflag=0;
	ts.c_cc[VMIN]=1;
	ts.c_cc[VTIME]=1;
	if(tcsetattr(fd,TCSANOW, &ts))
	{
		fprintf(stderr,"%s: can't set attributes of terminal\n",
			program_name);
		perror("tcsetattr");
		return -1;
	}
	return 0;
}


int set_termios_rts(int fd, int rts)
{
	/* ioctl() needs a pointer to int third argument, so here's the int. */
	static const int tiocm_rts=TIOCM_RTS;
	int result;
	if(debug & DEBUG_TERM)
	{
		fprintf(stderr,"set_termios_rts: fd %d, turning RTS %s\n",fd,rts?"on":"off");
	}
	result=ioctl(fd, rts?TIOCMBIS:TIOCMBIC, &tiocm_rts);
	if(result)
		perror("ioctl(TIOCM_RTS)");
	return result;
}


int set_termios_dtr(int fd, int dtr)
{
	/* ioctl() needs a pointer to int third argument, so here's the int. */
	static const int tiocm_dtr=TIOCM_DTR;
	int result;
	if(debug & DEBUG_TERM)
	{
		fprintf(stderr,"set_termios_dtr: fd %d, turning DTR %s\n",fd,dtr?"on":"off");
	}
	result=ioctl(fd, dtr?TIOCMBIS:TIOCMBIC, &tiocm_dtr);
	if(result)
		perror("ioctl(TIOCM_DTR)");
	return result;
}
